import axios from 'axios'
import store from '../store'
import { cookie } from '@/utils/cookie'
import { AESencrypt, MD5 } from '@/utils/crypto'
import { Message, MessageBox } from 'element-ui'

import NProgress from 'nprogress' // Progress 进度条
import 'nprogress/nprogress.css'// Progress 进度条样式
NProgress.configure({ showSpinner: false })

// 创建axios实例
const service = axios.create({
  _method: 'post',
  timeout: 20000,
  stateless: false, // 是否无code码
  encrypt: process.env.ENCRYPT, // 是否加密
  baseURL: process.env.BASE_URL, // api的base_url
  headers: { 'Content-Type': 'application/json' }
})

// 加载进度条req拦截器
service.interceptors.request.use(config => {
  NProgress.start()
  return config
}, err => {
  NProgress.done()
  Promise.reject(err)
})

// 超时req拦截器
service.interceptors.request.use(config => {
  config.now = Date.now()
  return config
}, err => Promise.reject(err))

// 下载req拦截器
service.interceptors.request.use(config => {
  if (config.download) {
    config.responseType = 'blob'
    config.timeout = 60000
  }
  return config
}, err => Promise.reject(err))

// 加密req拦截器
service.interceptors.request.use(config => {
  config.method = config._method
  config.headers['X-Token'] = cookie.get_xToken()
  if (config.encrypt) {
    const randomKey = cookie.get_randomKey()
    const objtokenect = cookie.get_token()
    const object = AESencrypt(JSON.stringify(config.data), cookie.get_aesKey())
    const sign = MD5(object + randomKey)
    config.headers['Authorization'] = 'Bearer ' + objtokenect
    config.data = JSON.stringify({ object, sign })
  }
  return config
}, err => Promise.reject(err))

// rep拦截器
service.interceptors.response.use(rep => {
  NProgress.done()
  // 下载rep拦截器
  if (rep.config.download) {
    const content = rep.data
    const blob = new Blob([content])
    const fileName = Date.now() + '.xls'
    if ('download' in document.createElement('a')) { // 非IE下载
      const elink = document.createElement('a')
      elink.download = fileName
      elink.style.display = 'none'
      elink.href = URL.createObjectURL(blob)
      document.body.appendChild(elink)
      elink.click()
      URL.revokeObjectURL(elink.href) // 释放URL 对象
      document.body.removeChild(elink)
    } else { // IE10+下载
      navigator.msSaveBlob(blob, fileName)
    }
    return rep
  }
  // 错误状态rep拦截器
  if (!rep.config.stateless && rep.data.code !== 200) {
    Message({
      message: `${rep.data.code} : ${rep.data.message}`,
      type: 'error',
      duration: 5 * 1000
    })
    return Promise.reject('error')
  }
  return rep
}, err => {
  NProgress.done()
  if (err.response) {
    // 下载rep拦截器
    if (err.config.download) {
      Message({
        type: 'error',
        duration: 5 * 1000,
        message: `下载失败`
      })
      return Promise.reject(err.response)
    }
    // 错误状态rep拦截器
    if (err.response.data.code === 700) {
      MessageBox.confirm('您已被登出，可以取消继续留在该页面，或者重新登录。', '确定登出', {
        confirmButtonText: '重新登录',
        cancelButtonText: '取消',
        type: 'warning'
      }).then(() => {
        store.dispatch('FedLogOut').then(() => {
          location.reload()// 为了重新实例化vue-router对象 避免bug
        })
      })
      return Promise.reject(err.response)
    } else if (err.response.data.code === 300 || err.response.data.message.indexOf('用户未登录') >= 0) {
      MessageBox.confirm('您还没有登陆，可以取消继续留在该页面，或者重新登录。', '确定登出', {
        confirmButtonText: '重新登录',
        cancelButtonText: '取消',
        type: 'warning'
      }).then(() => {
        store.dispatch('FedLogOut').then(() => {
          location.reload()// 为了重新实例化vue-router对象 避免bug
        })
      })
      return Promise.reject(err.response)
    } else {
      Message({
        type: 'error',
        duration: 5 * 1000,
        message: `${err.response.data.code} : ${err.response.data.message}`
      })
      return Promise.reject(err.response)
    }
  } else if (err.request) {
    // 超时rep拦截器
    if (err.request.readyState === 4 && err.request.status === 0) {
      if (Date.now() - err.config.now >= err.config.timeout) {
        Message({
          type: 'error',
          duration: 5 * 1000,
          message: `请求超时`
        })
      } else {
        Message({
          type: 'error',
          duration: 5 * 1000,
          message: `请求失败`
        })
      }
      return Promise.reject(err.response)
    }
  } else {
    return Promise.reject(err.response)
  }
})

export default service
